# JW 6/13/23

# If needed, 
install.packages("readtextgrid")
install.packages("reshape2")
install.packages("purrr")
install.packages("dplyr")
install.packages("stringr")
install.packages("data.table")

# load packages
library(readtextgrid)
library(reshape2)
library(purrr)
library(dplyr)
library(stringr)
library(data.table)

# Select the directory where the target TextGrid files are stored and set it as the working directory. This can be done in R Studio in the Session drop-down menu: Session > Set Working Directory > Choose Directory...

# save the working directory as a variable called 'path'.  
path <- getwd()

# Save the directory information as a variable called 'files'. The 'path' argument should contain the variable 'path', which contains the file trail of the folder where the target TextGrids are stored.   
files <- list.files(
  path = path, 
  pattern = "TextGrid$",
  full.names = TRUE, 
  recursive = TRUE
)

# Read in TextGrids using 'paths' variable and read_textgrid
tg <- map_dfr(files, read_textgrid)

# We are only interested segments containing labels, so we need to extract the values in the 'text' column that contain labels. Strip empty values from the dataframe. We should be left with ONLY the values beep-N1, beep-the, and full.sentence for each TextGrid file. 
tg <- tg %>% filter(str_detect(text, "[^[:space:]]"))
tg <- tg %>% mutate(across(where(is.character), str_trim))

# Create a variable called dur that captures the duration of each labeled segment.  
tg <- mutate(tg, dur = xmax - xmin)

# Check the levels of the 'text' column. There should be three unique labels (beep-N1, beep-the, and full.sentence). The script will still work if the labels have variations (e.g. BeepThe, fullSentence, etc.), but it will add additional columns. To make analysis easier, you may want to edit your labels if levels(factor(tg$text)) returns more than three levels.
levels(factor(tg$text))

# Ensure that all tiers have labeled segments that are a subset of the total duration of the tier. If a tier does not have a labelled segment (e.g., because the label was mistakenly left out), labeled segment duration should be set to 0. This step is intended to make it easy to detect labeling mistakes. Segments that are not labelled will have a duration of 0.
setDT(tg)[dur == tier_xmax, dur := 0]

# Transform the data to wide format 
tg2 <- dcast(tg, file ~ text, value.var = "dur")

# convert all columns to milliseconds
tg2[,2:length(names(tg2))] <- tg2[,2:length(names(tg2))]*1000

# Write a .csv containing the labeled segments and their duration for each TextGrid file, and put it in the directory containing the sound files and text grids. 
write.csv(tg2, paste(path,'/Response time.csv',sep = ''), row.names = FALSE)

